load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/shea_util.ncl"

function make_mask(v[*][*]:numeric)
begin

; Function which creates a scalar array from meridional velocity
; The scalar has the same size as V
; Values of 1 if northward
; Values of 0 if southward

v1d = ndtooned(v)
testtemp = new(dimsizes(v1d), typeof(v1d))
ipos = ind(v1d.ge.0)
ineg = ind(v1d.lt.0)
testtemp(ipos)=1
testtemp(ineg)=-1
testout = onedtond(testtemp, dimsizes(v))
return testout

end

begin

; map coordinates
lonmin=-80
lonmax=0
latmin=-25
latmax=70

; reading the values of drifter velocities on the subdomain
fin = addfile("data/drifter_annualmeans.nc", "r")
u = fin->U({latmin:latmax},{lonmin:lonmax})
v = fin->V({latmin:latmax},{lonmin:lonmax})
; masking bad drifter data
u = mask(u, abs(u).ge.800, False)
v = mask(v, abs(v).ge.800, False)
; creation of the test array (1 when v>0, -1 if v<0)
test = make_mask(v)
delete(fin)

; opening of the Andro file
; converting from dat to nc by using Python
; same as previously
fin = addfile("data/deepVel.nc", "r")
uatmp = fin->ud(lat|:, lon|:)
vatmp = fin->vd(lat|:, lon|:)
lon = fin->Xd
lat = fin->Yd
uatmp&lon = lon
uatmp&lat = lat
vatmp&lon = lon
vatmp&lat = lat
ua = uatmp({latmin:latmax},{lonmin:lonmax})
va = vatmp({latmin:latmax},{lonmin:lonmax})
printVarSummary(ua)
printVarSummary(va)


ua = mask(ua, abs(ua).ge.800, False)
va = mask(va, abs(va).ge.800, False)
testa = make_mask(va)
delete(fin)

if (False) then
    ; reading of the bathy file
    ; this is an old file, replace with yours
    fin = addfile("data/bathymetry.nc", "r")
    bathytmp = fin->bath
    bathytmp = lonFlip(bathytmp)
    bathy = bathytmp({latmin:latmax},{lonmin:lonmax})
    bathy = mask(bathy, bathy.ge.0, False)
else
    print("Smooth")
    fin = addfile("data/bathAtl.nc", "r")
    bathytmp = fin->dep
    lonbat = fin->longD(lat|:, lon|:)
    latbat = fin->latD(lat|:, lon|:)
    bathy = bathytmp(lat|:, lon|:)
    bathy = mask(bathy, bathy.ge.0, False)
end if 


; opening of a workspace (format, name), format=pdf, png, etc
wks = gsn_open_wks("pdf", "martha")
; definition of an homemade colormap.
; background, forground, south arrows, north arrows, bathy
;cmap = (/"white", "black", "blue", "red", "gray50", "gray60", "gray70", "gray80", "gray90", "white"/)
cmap = (/"white", "black", "blue", "red", "gray60", "gray70", "gray80", "gray90", "white"/)
gsn_define_colormap(wks, cmap)

; Defining the plot resources for the Bathymetry plot
; resbat also contains the resources for the map
resbat=True
resbat@gsnAddCyclic=False ; no cyclic data
resbat@gsnDraw=False ; no drawing 
resbat@gsnFrame=False ; no framing

; Defining the map domain
resbat@mpLimitMode            = "LatLon" ; LatLon limit mode
resbat@mpMinLatF=latmin 
resbat@mpMaxLatF=latmax
resbat@mpMinLonF=lonmin
resbat@mpMaxLonF=lonmax

; We fill the continents and draw the coastlines
resbat@mpFillOn               = True ; filled map
resbat@mpLandFillColor               = "black" ; filled map
resbat@mpOutlineOn            = True  ; continental lines
resbat@mpOutlineDrawOrder     = "PostDraw" ; the map is drawn last  
resbat@mpGridAndLimbOn        =  False ; we draw the grid
resbat@mpGridLineDashPattern      = 2 ; dashed grid
resbat@mpGridLatSpacingF = 10 ; grid spacing
resbat@mpGridLonSpacingF = 10

; Ncl recover the variable attributes and use
; them as left and right title -> we remove the titles
resbat@gsnLeftString=""
;resbat@gsnCenterString=""
resbat@gsnRightString=""

; X/Y contourf coordinates
resbat@sfXArray = lonbat
resbat@sfYArray = latbat

; force filled contour, explicit levels, from 0 to 5000 by 10000
resbat@cnFillOn=True
resbat@cnLevelSelectionMode="ExplicitLevels"
resbat@cnLevels=ispan(0,5000, 1000)
resbat@lbLabelBarOn=False ; no label bar
resbat@cnFillMode="AreaFill" ; contourf (CellFill=pcolor, RasterFill=raster)

; Force to use all the colormap
; We use the 4th color as the first one for the contourf
; and the 10th as the last one
resbat@gsnSpreadColors=True
resbat@gsnSpreadColorStart=10
resbat@gsnSpreadColorEnd=4

; No contour lines and no contour labels
resbat@cnLinesOn=False
resbat@cnLineLabelsOn=False

; changing the size of the labels
resbat@tmXBLabelFontHeightF=0.012
resbat@tmYLLabelFontHeightF=0.012

resbatlines = resbat
delete(resbatlines@cnLevels)
resbatlines@cnLevels=ispan(0, 5000, 1000)
resbatlines@cnLinesOn=True
resbatlines@cnFillOn=False
resbatlines@cnLineColor="white"
resbatlines@cnInfoLabelOn=False
resbatlines@cnLineThicknessF=0.1

; Defining the resources for the vector plot

res=True

; same as previously
res@gsnFrame=False
res@gsnDraw = False
res@gsnAddCyclic = False

; arrow coordinates
res@vfXArray = u&Lon ; arrows coordinates
res@vfYArray = u&Lat 

; activat curly vectors
res@vcGlyphStyle            = "CurlyVector" 
; minimum distance between arrows
res@vcMinDistanceF          = 0.013  

; length of the reference arrow
res@vcRefLengthF            = 0.025  ; reference length  

; thickness of the arrows
res@vcLineArrowThicknessF = 2.0

; Settings for the arrow colors
; Same thing as for the contourf
res@gsnSpreadColors=True
res@gsnSpreadColorEnd=3
res@gsnSpreadColorStart=2
res@lbLabelBarOn=False

; Settings for the reference arrow
res@vcRefAnnoFontHeightF = 0.018 ; label size
res@vcRefAnnoOn=True ; turn label on
res@vcRefAnnoString2On = False ; turn default string off
; Changing the position of the reference position
res@vcRefAnnoOrthogonalPosF   = -0.53 
res@vcRefAnnoParallelPosF   = 0.96
; adding some padding to the arrow
res@vcRefAnnoPerimSpaceF=0.33 ; label box space 
res@vcRefAnnoArrowSpaceF=0.7

; magnitude of the reference arrow
; value of refmag will have a lengh of vcRefLengthF
refmag = 20  ; reference magnitude of the arrow
res@vcRefMagnitudeF         = refmag    
; string of the reference arrow
res@vcRefAnnoString1 =refmag +" cm/s" ; reference label
; adding the title to the RESBAT resource
resbat@gsnCenterString="Currents at the surface from drifters"

lonbmin = -40.5
lonbmax = -54.5 
latbmin = 37.5 
latbmax = 47.5
lonbox = (/lonbmin, lonbmax, lonbmax, lonbmin, lonbmin/)
latbox = (/latbmin, latbmin, latbmax, latbmax, latbmin/)
resbox = True
resbox@gsLineThicknessF = 4.0

; Drawing the contour map
plotbat1 = gsn_csm_contour_map(wks, -bathy, resbat)
plotbatlines1 = gsn_csm_contour(wks, -bathy, resbatlines)
overlay(plotbat1, plotbatlines1)
; Drawing the coloured arrows (NO MAP!!!)
plot1=gsn_csm_vector_scalar(wks,u*100,v*100,test, res) 
; overlay the arrows on the contour map
overlay(plotbat1, plot1)
; delete the vfArrays 
delete(res@vfXArray)
delete(res@vfYArray)
dum1 = gsn_add_polyline(wks, plotbat1, lonbox, latbox, resbox) 

; changing the resources for the next plot
refmag = 4
res@vfXArray=ua&lon
res@vfYArray=ua&lat
res@vcRefMagnitudeF         = refmag      ; reference magnitude         
res@vcRefAnnoString1 =refmag +" cm/s" ; reference label
resbat@gsnCenterString="Currents at 1000 m from Argo"
plotbat2 = gsn_csm_contour_map(wks, -bathy, resbat)
plotbatlines2 = gsn_csm_contour(wks, -bathy, resbatlines)
overlay(plotbat2, plotbatlines2)
plot2=gsn_csm_vector_scalar(wks,ua,va,testa, res) 
overlay(plotbat2, plot2)
dum2 = gsn_add_polyline(wks, plotbat2, lonbox, latbox, resbox) 

; defining a plot array for panelling
plotF=new(2,graphic)
plotF(0) = plotbat1
plotF(1) = plotbat2

; panelling resources
resP=True
resP@gsnPanelFigureStrings = (/"a)","b)"/)
; changing the position of the strings
resP@amParallelPosF = -0.3
resP@amOrthogonalPosF = 0.3
; changing the size of strings
resP@gsnPanelFigureStringsFontHeightF = 0.016

; we draw the panels (it does a Draw and a Frame automatically)
gsn_panel(wks, plotF, (/1,2/), resP)

; uncomment the next line to check color numbers on the cmap
;gsn_draw_colormap(wks)

end
